"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const MemcachedPlus = require("@vingle/memcache-plus");
class MemcachedDriver {
    constructor(serverURL, options) {
        this.serverURL = serverURL;
        this.options = options;
        this.client = new MemcachedPlus({
            hosts: [serverURL],
            autodiscover: options.autoDiscovery || false,
            netTimeout: 500,
            reconnect: true,
        });
    }
    get(key) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.client.get(key);
            if (res === null) {
                return undefined;
            }
            else {
                return res;
            }
        });
    }
    getMulti(keys) {
        return __awaiter(this, void 0, void 0, function* () {
            const values = yield this.client.getMulti(keys);
            const res = {};
            for (const key in values) {
                const value = values[key];
                res[key] = value;
            }
            return res;
        });
    }
    set(key, value, lifetime) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.client.set(key, value, lifetime);
        });
    }
    del(key) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.client.delete(key);
        });
    }
    flush() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.client.flush();
        });
    }
}
exports.MemcachedDriver = MemcachedDriver;
//# sourceMappingURL=memcached.js.map