"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const crypto = require("crypto");
function isNotUndefinedOrNull(x) {
    return x !== undefined && x !== null;
}
class MemcachedFetcher {
    constructor(driver, options = {}) {
        this.driver = driver;
        const keyHashing = options.keyHashing !== undefined ? options.keyHashing : true;
        if (keyHashing) {
            this.keyHasher = (key) => {
                return crypto
                    .createHash("md5")
                    .update(key)
                    .digest("hex");
            };
        }
        else {
            this.keyHasher = (key) => key;
        }
    }
    fetch(key, lifetime, fetcher) {
        return __awaiter(this, void 0, void 0, function* () {
            const hasheKey = this.keyHasher(key);
            let value = yield this.driver.get(hasheKey);
            if (!isNotUndefinedOrNull(value)) {
                value = yield fetcher();
                yield this.driver.set(hasheKey, value, lifetime);
                return value;
            }
            return value;
        });
    }
    del(key) {
        return __awaiter(this, void 0, void 0, function* () {
            const hasheKey = this.keyHasher(key);
            return yield this.driver.del(hasheKey);
        });
    }
    multiFetch(args, key, lifetime, fetcher) {
        return __awaiter(this, void 0, void 0, function* () {
            if (args.length === 0) {
                return [];
            }
            const { namespace, argToKey } = (() => {
                if (typeof key === "string") {
                    return { namespace: key, argToKey: (arg) => String(arg) };
                }
                else {
                    return { namespace: key[0], argToKey: key[1] };
                }
            })();
            const argsToKeyMap = new Map(args.map((arg) => {
                const key = this.keyHasher(`${namespace}:${argToKey(arg).toString()}`);
                return [arg, key];
            }));
            const cached = yield this.driver.getMulti(Array.from(argsToKeyMap.values()));
            const missingArgs = args.filter((arg) => !isNotUndefinedOrNull(cached[argsToKeyMap.get(arg)]));
            const fetchedArray = missingArgs.length > 0 ? yield fetcher(missingArgs) : [];
            if (fetchedArray.length !== missingArgs.length) {
                throw new Error("Fetcher must return same length of result with Args.length");
            }
            const fetched = new Map(missingArgs.map((arg, index) => [arg, fetchedArray[index]]));
            yield Promise.all(Array.from(fetched).map(([arg, result]) => __awaiter(this, void 0, void 0, function* () {
                yield this.driver.set(argsToKeyMap.get(arg), result, lifetime);
            })));
            return args.map((arg) => {
                const key = argsToKeyMap.get(arg);
                if (isNotUndefinedOrNull(cached[key])) {
                    return cached[key];
                }
                else {
                    return fetched.get(arg);
                }
            });
        });
    }
    multiFetchDelete(args, key) {
        return __awaiter(this, void 0, void 0, function* () {
            const { namespace, argToKey } = (() => {
                if (typeof (key) === "string") {
                    return { namespace: key, argToKey: (arg) => arg.toString() };
                }
                else {
                    return { namespace: key[0], argToKey: key[1] };
                }
            })();
            yield Promise.all(args.map((arg) => __awaiter(this, void 0, void 0, function* () {
                const key = this.keyHasher(`${namespace}:${argToKey(arg).toString()}`);
                yield this.driver.del(key);
            })));
        });
    }
}
exports.MemcachedFetcher = MemcachedFetcher;
//# sourceMappingURL=fetcher.js.map